//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ENBSeries effect file
// visit http://enbdev.com for updates
// Copyright (c) 2007-2013 Boris Vorontsov
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



#define DOFQuality 		4
#define DOFBlurRadius 		8.5
#define DOFCurve 		8.0
#define DOFSmoothening  	1.0


//+++++++++++++++++++++++++++++
//external parameters, do not modify
//+++++++++++++++++++++++++++++
//keyboard controlled temporary variables (in some versions exists in the config file). Press and hold key 1,2,3...8 together with PageUp or PageDown to modify. By default all set to 1.0
float4	tempF1; //0,1,2,3
float4	tempF2; //5,6,7,8
float4	tempF3; //9,0
//x=Width, y=1/Width, z=ScreenScaleY, w=1/ScreenScaleY
float4	ScreenSize;
//x=generic timer in range 0..1, period of 16777216 ms (4.6 hours), w=frame time elapsed (in seconds)
float4	Timer;
//changes in range 0..1, 0 means that night time, 1 - day time
float	ENightDayFactor;
//changes 0 or 1. 0 means that exterior, 1 - interior
float	EInteriorFactor;
//adaptation delta time for focusing
float	FadeFactor;
//fov in degrees
float	FieldOfView;
//transposed transform matrix view*projection and inverse of it
float4	MatrixVP[4];
float4	MatrixInverseVP[4];

//textures
texture2D texColor;
texture2D texDepth;
texture2D texNoise;
texture2D texPalette;
texture2D texFocus;
texture2D texCurr; 
texture2D texPrev; 

sampler2D SamplerColor = sampler_state
{
	Texture   = <texColor>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;//NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerDepth = sampler_state
{
	Texture   = <texDepth>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerNoise = sampler_state
{
	Texture   = <texNoise>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = NONE;//NONE;
	AddressU  = Wrap;
	AddressV  = Wrap;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerPalette = sampler_state
{
	Texture   = <texPalette>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;//NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerCurr = sampler_state
{
	Texture   = <texCurr>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;//NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerPrev = sampler_state
{
	Texture   = <texPrev>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

sampler2D SamplerFocus = sampler_state
{
	Texture   = <texFocus>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = NONE;
	AddressU  = Clamp;
	AddressV  = Clamp;
	SRGBTexture=FALSE;
	MaxMipLevel=0;
	MipMapLodBias=0;
};

struct VS_OUTPUT_POST
{
	float4 vpos  : POSITION;
	float2 txcoord : TEXCOORD0;
};

struct VS_INPUT_POST
{
	float3 pos  : POSITION;
	float2 txcoord : TEXCOORD0;
};

VS_OUTPUT_POST VS_PostProcess(VS_INPUT_POST IN)
{
	VS_OUTPUT_POST OUT;

	float4 pos=float4(IN.pos.x,IN.pos.y,IN.pos.z,1.0);

	OUT.vpos=pos;
	OUT.txcoord.xy=IN.txcoord.xy;

	return OUT;
}

float4 PS_ReadFocus(VS_OUTPUT_POST IN) : COLOR
{
	return tex2D(SamplerDepth,0.5).xxxx;
}

float4 PS_WriteFocus(VS_OUTPUT_POST IN) : COLOR
{
	float res = 0.0;
	float curr = tex2D(SamplerCurr, 0.5).x;
	float prev = tex2D(SamplerPrev, 0.5).x;

	res = lerp(prev, curr, saturate(FadeFactor));	
	res = max(res, 0.0);

	return res;
}

technique ReadFocus
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_PostProcess();
		PixelShader  = compile ps_3_0 PS_ReadFocus();

		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		AlphaBlendEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}
}

technique WriteFocus
{
	pass P0
	{
		VertexShader = compile vs_3_0 VS_PostProcess();
		PixelShader  = compile ps_3_0 PS_WriteFocus();

		ZEnable=FALSE;
		CullMode=NONE;
		ALPHATESTENABLE=FALSE;
		SEPARATEALPHABLENDENABLE=FALSE;
		AlphaBlendEnable=FALSE;
		FogEnable=FALSE;
		SRGBWRITEENABLE=FALSE;
	}
}


float4 PS_ProcessPass1(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR
{
	float	scenedepth=tex2D(SamplerDepth, IN.txcoord.xy);
	float	scenefocus=tex2D(SamplerFocus, 0.5).x - 0.07;
	float CoC = smoothstep(0.07, 0.25, abs(scenedepth - scenefocus));
	float2 discRadius = DOFBlurRadius * CoC * float2(ScreenSize.y,ScreenSize.y*ScreenSize.z);
	
	float4 res = tex2D(SamplerColor, IN.txcoord.xy);
	res = float4(pow(res.xyz,DOFCurve)*0.2,0.2);
	const int rings = round(lerp(1.0,DOFQuality,CoC));

	[loop]
	for(int i=1;i<=rings;i++)
	{
		float radiusCoeff = (float)i/(float)rings;

		int ringsamples = 4 * i + 2;

		[loop]
		for(int j=0; j < ringsamples && j < 34; j++)
		{
			float2 tapoffset = 0.0;
			sincos(6.283*(float)j/ringsamples,tapoffset.y,tapoffset.x);
			res.xyz += pow(tex2Dlod(SamplerColor, float4(IN.txcoord.xy + tapoffset.xy * discRadius.xy * radiusCoeff,0,0)).xyz,DOFCurve);
			res.w++;
		}
	}

	res.xyz /= res.w;

	res.xyz = pow(res.xyz,1.0/DOFCurve);

	res.w=CoC;
	return res;
}

float4 PS_ProcessPass2(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR
{
	float4 res = tex2D(SamplerColor, IN.txcoord.xy);
	float2 PixelSize = float2(ScreenSize.y,ScreenSize.y*ScreenSize.z);
	float blurradius = res.w;
	blurradius = smoothstep(0.0,0.5,blurradius)*DOFSmoothening;

	float gaussweight[4] = {0.195,0.122,0.054,0.016};

	res.xyz *= 0.227;

	for(int i=0; i<4; i++)
	{
		res.xyz += tex2Dlod(SamplerColor, float4(IN.txcoord.xy + float2(PixelSize.x,0.0) * blurradius * ((float)i+1),0,0)).xyz * gaussweight[i];
		res.xyz += tex2Dlod(SamplerColor, float4(IN.txcoord.xy - float2(PixelSize.x,0.0) * blurradius * ((float)i+1),0,0)).xyz * gaussweight[i];
	}

	return res;
}

float4 PS_ProcessPass3(VS_OUTPUT_POST IN, float2 vPos : VPOS) : COLOR
{
	float4 res = tex2D(SamplerColor, IN.txcoord.xy);
	float2 PixelSize = float2(ScreenSize.y,ScreenSize.y*ScreenSize.z);
	float blurradius = res.w;
	blurradius = smoothstep(0.0,0.5,blurradius)*DOFSmoothening;

	float gaussweight[4] = {0.195,0.122,0.054,0.016};

	res.xyz *= 0.227;

	for(int i=0; i<4; i++)
	{
		res.xyz += tex2Dlod(SamplerColor, float4(IN.txcoord.xy + float2(0.0,PixelSize.y) * blurradius * ((float)i+1),0,0)).xyz * gaussweight[i];
		res.xyz += tex2Dlod(SamplerColor, float4(IN.txcoord.xy - float2(0.0,PixelSize.y) * blurradius * ((float)i+1),0,0)).xyz * gaussweight[i];
	}

	return saturate(res);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
technique PostProcess
{
	pass P0
	{

		VertexShader = compile vs_3_0 VS_PostProcess();
		PixelShader  = compile ps_3_0 PS_ProcessPass1();
	}
}

technique PostProcess2
{
	pass P0
	{

		VertexShader = compile vs_3_0 VS_PostProcess();
		PixelShader  = compile ps_3_0 PS_ProcessPass2();
	}
}

technique PostProcess3
{
	pass P0
	{

		VertexShader = compile vs_3_0 VS_PostProcess();
		PixelShader  = compile ps_3_0 PS_ProcessPass3();
	}
}