//SM2 Null material (vertex shader), with reflections. As it is SM2 (not 2a+), there is no normal map
//SM2a+ Null materials will have normal map support.

// This is used by 3dsmax to load the correct parser
string ParamID = "0x003";

//////////////////////////////////////////////////
//Matrices
//////////////////////////////////////////////////
half4x4 WorldIT : WorldInverseTranspose < string UIWidget = "None"; >;
half4x4 WorldViewProj : WorldViewProjection < string UIWidget = "None"; >;
half4x4 World : World < string UIWidget = "None"; >;
half4x4 ViewInv : ViewInverse < string UIWidget = "None"; >;

//////////////////////////////////////////////////
//Point Light structure. Sun is hardcoded.
//This works because every light created from the map, gunfire, explosions, etc, is *supposed* to be a pointlight
//and infact, IS!
//////////////////////////////////////////////////
struct pointlight
{
	half4 lPosition ;//store Attn in .w component
	half4 lColor;
};

#define maxlights 1
int activelights = 0;//should not exceed maxlights

pointlight light[maxlights] =
{
	
	{
		half4(5.0f, 1.0f, 1.0f,1.0f),
		half4(0.0f, 1.0f, 0.0f,0.0f),
		//half4(1.0f, 1.0f, 1.0f,1.0f),
		//half4(0.0f, 0.0f, 1.0f,0.0f),
	}
};

//////////////////////////////////////////////////
//Main Light
//////////////////////////////////////////////////
half3 LightDir : Direction
<
	string Object = "Spotlight";
	string Space = "World";
> = {1.0f, -1.0f, 1.0f};

half4 LightColor1 : Diffuse <
    string UIName =  "Light1 Color";
    string Object = "Spotlight";
    string UIWidget = "Color";
> = {1.0f, 1.0f, 1.0f, 1.0f};

//////////////////////////////////////////////////
//Material surface
//////////////////////////////////////////////////
half4 MatDiffuse : Diffuse
<
    string UIName =  "Material Diffuse";
    string UIWidget = "Color";
> = {1.0f, 1.0f, 1.0f, 1.0f};

half4 MatAmbient : Ambient
<
    string UIName =  "Material Ambient";
    string UIWidget = "Color";
> = {0.0f, 0.0f, 0.0f, 1.0f};

half4 MatSpec : Specular
<
    string UIName =  "Material Specular";
    string UIWidget = "Color";
> = {1.0f, 1.0f, 1.0f, 1.0f};

half MatSpecPow : SpecularPower <
    string UIWidget = "slider";
    half UIMin = 1.0;
    half UIMax = 128.0;
    half UIStep = 1.0;
    string UIName =  "specular power";
> = 40.0;

half refmult : Environment
<
    string UIWidget = "slider";
    half UIMin = 0.0f;
    half UIMax = 2.0f;
    half UIStep = 0.001f;
    string UIName = "Reflection Multiplier";
> = 0.5f;

half4 sceneambient
<
    string UIName =  "Scene Ambient";
    string UIWidget = "Color";
> = {0.0f, 0.0f, 0.0f, 1.0f};

//////////////////////////////////////////////////
//Textures
//////////////////////////////////////////////////

texture colorTexture : Diffuse <
    string ResourceName = "default_color.dds";
    string ResourceType = "2D";
>;

texture refcubeTexture : Environment <
    string ResourceName = "default_reflection.dds";
    string ResourceType = "CUBE";
>;

texture normalTexture : Normal <
    string ResourceName = "default_bump_normal.dds";
    string ResourceType = "2D";
>;

//////////////////////////////////////////////////
//Samplers
//////////////////////////////////////////////////

sampler2D colorSampler = sampler_state {
	Texture = <colorTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
};

samplerCUBE envSampler = sampler_state {
	Texture = <refcubeTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
};

sampler2D normalSampler = sampler_state {
	Texture = <normalTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
};

//////////////////////////////////////////////////
//Structures
//////////////////////////////////////////////////

//Vertex structure from game
struct appdata {
    half3 Position	: POSITION;
    half3 UV		: TEXCOORD0;
    half4 Normal	: NORMAL;
    half4 Binormal	: BINORMAL0;
    half4 Tangent	: TANGENT0;
};

//data passed from the vertex shader to the pixel shader
struct vertexOutput {
    half4 HPosition	: POSITION;
    half3 TexCoord	: TEXCOORD0;
    half3 LightVec1	: TEXCOORD1;
    half3 WorldNormal	: TEXCOORD2;
    half3 WorldPos	: TEXCOORD3;
    half3 WorldEyePos	: TEXCOORD4;
    half3 WorldTangent	: TEXCOORD5;
    half3 WorldBinorm	: TEXCOORD6;
};

//////////////////////////////////////////////////
//Shaders
//////////////////////////////////////////////////

//Vertex Shader
vertexOutput mainVS(appdata IN)
{	
	vertexOutput OUT;
    OUT.WorldNormal = mul(IN.Normal, WorldIT).xyz;
    half4 Po = half4(IN.Position.xyz,1.0);
    half3 Pw = mul(Po, World).xyz;
    OUT.WorldPos = Pw;
    OUT.LightVec1 = normalize(-LightDir);
    OUT.TexCoord = IN.UV;
    OUT.WorldEyePos = ViewInv[3].xyz;
    OUT.HPosition = mul(Po, WorldViewProj);
    OUT.WorldTangent = mul(IN.Tangent, WorldIT).xyz;
    OUT.WorldBinorm = mul(IN.Binormal, WorldIT).xyz;
    return OUT;
}

//Pixel Shader
half4 mainPS(vertexOutput IN) : COLOR
{
    half4 map = tex2D(colorSampler,IN.TexCoord.xy);
    half3 Nn = normalize(IN.WorldNormal);
    half3 Vn = normalize(IN.WorldEyePos - IN.WorldPos);
    //normal map
    half3 bumps = tex2D(normalSampler,IN.TexCoord.xy).xyz - (0.5).xxx;
    half3 Tn = normalize(IN.WorldTangent);
    half3 Bn = normalize(IN.WorldBinorm);
    half3 Nb = Nn + (bumps.x * Tn + bumps.y * Bn);
    Nb = normalize(Nb);
    
    //calc light vec
    half3 Ln = normalize(IN.LightVec1.xyz);
    half3 Hn = normalize(Vn + Ln);
    half hdn = pow(max(0,dot(Hn,Nb)),MatSpecPow);// * 1.0f;
    half ldn = dot(Ln,Nb);
    ldn = max(0.0,ldn);
    
    //blend sun light color
    half4 diffContrib = ldn * LightColor1;
    half4 specContrib = (ldn * hdn) * (LightColor1 * MatSpec);
    
    /*       
    for(int i = 0; i < activelights; i++)
   	{
 		//calc light vec2 - shitty, no attenuation yet
		half3 Ln2 = normalize(light[i].lPosition);
		//half3 Ln2 = (light[i].lPosition - Vn);
		//half Ld1 = 1.0/length(Ln2);
		half3 Hn2 = normalize(Vn + Ln2);
  		half hdn2 = pow(max(0,dot(Hn2,Nb)),MatSpecPow);// * 1.0f;
  		half ldn2 = light[i].lPosition.w * dot(Ln2,Nb);
		ldn2 = max(0.0,ldn);
    		
		//blend light colors
		diffContrib += ldn2 * light[i].lColor;
 		specContrib += (ldn2 * hdn2) * (light[i].lColor * MatSpec);
   	}
   	*/
    
	//reflection map coordinates are flipped and swizzled for GTA
	half3 refvect = reflect(Vn,Nb);
	refvect.xyz = refvect.xzy;//this is expensive. it's either this, or multiple lights, but not both (under sm20)
	half4 reflColor = texCUBE(envSampler,-refvect.xyz);
	
	//blend material diffuse, light color and ambient, add reflective light
	//half4 result = (map * MatDiffuse) * (diffContrib + MatAmbient + sceneambient) + (specContrib + (reflColor * refmult)); 
	half4 result = ((MatAmbient + sceneambient) + (MatDiffuse * diffContrib)) * map + (specContrib + (reflColor * refmult));    
	result.a = MatDiffuse.a * map.a;
	
	return result;
}

//////////////////////////////////////////////////
//Technique - not used by game
//////////////////////////////////////////////////

technique Bumpref
{
	pass p0
	{
		AlphaBlendEnable = TRUE;
		AlphaTestEnable = TRUE;
		AlphaFunc = Greater;
		AlphaRef = 0x01;
		SrcBlend = 5;
		DestBlend = 6;
		cullmode = None;
		VertexShader = compile vs_2_0 mainVS();
		PixelShader = compile ps_2_a mainPS();
	}
}